# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/afsproxy/v1/afsproxy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x61pi/afsproxy/v1/afsproxy.proto\x12\x0b\x61\x66sproxy.v1\"E\n\rStatusRequest\x12\x1b\n\x13include_layerstores\x18\x01 \x01(\x08\x12\x17\n\x0finclude_afslets\x18\x02 \x01(\x08\"\x91\x01\n\x1cReconcileImageReplicaRequest\x12\r\n\x05image\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x13\n\x0bplatform_os\x18\x03 \x01(\t\x12\x15\n\rplatform_arch\x18\x04 \x01(\t\x12\x18\n\x10platform_variant\x18\x05 \x01(\t\x12\x0f\n\x07replica\x18\x06 \x01(\x05\"w\n\x1dReconcileImageReplicaResponse\x12\x11\n\timage_key\x18\x01 \x01(\t\x12\x17\n\x0f\x63urrent_replica\x18\x02 \x01(\x05\x12\x19\n\x11requested_replica\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nsured\x18\x04 \x01(\x08\"w\n\rStatusSummary\x12\x1c\n\x14layerstore_instances\x18\x01 \x01(\x03\x12\x18\n\x10\x61\x66slet_instances\x18\x02 \x01(\x03\x12\x18\n\x10\x61\x66slet_reachable\x18\x03 \x01(\x03\x12\x14\n\x0ctotal_layers\x18\x04 \x01(\x03\"\xa7\x01\n\x12LayerstoreInstance\x12\x0f\n\x07node_id\x18\x01 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x02 \x01(\t\x12\x16\n\x0elast_seen_unix\x18\x03 \x01(\x03\x12\x17\n\x0f\x63\x61\x63he_max_bytes\x18\x04 \x01(\x03\x12&\n\x06layers\x18\x05 \x03(\x0b\x32\x16.afsproxy.v1.LayerInfo\x12\x15\n\rcached_images\x18\x06 \x03(\t\"-\n\tLayerInfo\x12\x0e\n\x06\x64igest\x18\x01 \x01(\t\x12\x10\n\x08\x61\x66s_size\x18\x02 \x01(\x03\"\xfc\x01\n\x0e\x41\x66sletInstance\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12\x11\n\treachable\x18\x02 \x01(\x08\x12\r\n\x05\x65rror\x18\x03 \x01(\t\x12\x1a\n\x12running_containers\x18\x04 \x01(\x03\x12\x17\n\x0flimit_cpu_cores\x18\x05 \x01(\x03\x12\x17\n\x0flimit_memory_mb\x18\x06 \x01(\x03\x12\x16\n\x0eused_cpu_cores\x18\x07 \x01(\x03\x12\x16\n\x0eused_memory_mb\x18\x08 \x01(\x03\x12\x1b\n\x13\x61vailable_cpu_cores\x18\t \x01(\x03\x12\x1b\n\x13\x61vailable_memory_mb\x18\n \x01(\x03\".\n\x0bStatusError\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xdb\x01\n\x0eStatusResponse\x12-\n\x07summary\x18\x01 \x01(\x0b\x32\x1a.afsproxy.v1.StatusSummaryH\x00\x12\x35\n\nlayerstore\x18\x02 \x01(\x0b\x32\x1f.afsproxy.v1.LayerstoreInstanceH\x00\x12-\n\x06\x61\x66slet\x18\x03 \x01(\x0b\x32\x1b.afsproxy.v1.AfsletInstanceH\x00\x12)\n\x05\x65rror\x18\x04 \x01(\x0b\x32\x18.afsproxy.v1.StatusErrorH\x00\x42\t\n\x07payload2\xbf\x01\n\x08\x41\x66sProxy\x12\x43\n\x06Status\x12\x1a.afsproxy.v1.StatusRequest\x1a\x1b.afsproxy.v1.StatusResponse0\x01\x12n\n\x15ReconcileImageReplica\x12).afsproxy.v1.ReconcileImageReplicaRequest\x1a*.afsproxy.v1.ReconcileImageReplicaResponseB2Z0github.com/reyoung/afs/pkg/afsproxypb;afsproxypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'api.afsproxy.v1.afsproxy_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z0github.com/reyoung/afs/pkg/afsproxypb;afsproxypb'
  _globals['_STATUSREQUEST']._serialized_start=47
  _globals['_STATUSREQUEST']._serialized_end=116
  _globals['_RECONCILEIMAGEREPLICAREQUEST']._serialized_start=119
  _globals['_RECONCILEIMAGEREPLICAREQUEST']._serialized_end=264
  _globals['_RECONCILEIMAGEREPLICARESPONSE']._serialized_start=266
  _globals['_RECONCILEIMAGEREPLICARESPONSE']._serialized_end=385
  _globals['_STATUSSUMMARY']._serialized_start=387
  _globals['_STATUSSUMMARY']._serialized_end=506
  _globals['_LAYERSTOREINSTANCE']._serialized_start=509
  _globals['_LAYERSTOREINSTANCE']._serialized_end=676
  _globals['_LAYERINFO']._serialized_start=678
  _globals['_LAYERINFO']._serialized_end=723
  _globals['_AFSLETINSTANCE']._serialized_start=726
  _globals['_AFSLETINSTANCE']._serialized_end=978
  _globals['_STATUSERROR']._serialized_start=980
  _globals['_STATUSERROR']._serialized_end=1026
  _globals['_STATUSRESPONSE']._serialized_start=1029
  _globals['_STATUSRESPONSE']._serialized_end=1248
  _globals['_AFSPROXY']._serialized_start=1251
  _globals['_AFSPROXY']._serialized_end=1442
# @@protoc_insertion_point(module_scope)
