# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/afslet/v1/afslet.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x61pi/afslet/v1/afslet.proto\x12\x08\x61\x66sletpb\"\xcb\x01\n\x0e\x45xecuteRequest\x12\'\n\x05start\x18\x01 \x01(\x0b\x32\x16.afsletpb.StartRequestH\x00\x12.\n\nfile_begin\x18\x02 \x01(\x0b\x32\x18.afsletpb.FileEntryBeginH\x00\x12)\n\nfile_chunk\x18\x03 \x01(\x0b\x32\x13.afsletpb.FileChunkH\x00\x12*\n\x08\x66ile_end\x18\x04 \x01(\x0b\x32\x16.afsletpb.FileEntryEndH\x00\x42\t\n\x07payload\"\xf6\x02\n\x0cStartRequest\x12\r\n\x05image\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x0f\n\x07\x63ommand\x18\x03 \x03(\t\x12\x11\n\tcpu_cores\x18\x04 \x01(\x03\x12\x11\n\tmemory_mb\x18\x05 \x01(\x03\x12\x12\n\ntimeout_ms\x18\x06 \x01(\x03\x12\x16\n\x0e\x64iscovery_addr\x18\x07 \x01(\t\x12\x19\n\x11\x66orce_local_fetch\x18\x08 \x01(\x08\x12\x0f\n\x07node_id\x18\t \x01(\t\x12\x13\n\x0bplatform_os\x18\n \x01(\t\x12\x15\n\rplatform_arch\x18\x0b \x01(\t\x12\x18\n\x10platform_variant\x18\x0c \x01(\t\x12\"\n\x1aproxy_dispatch_max_retries\x18\r \x01(\x03\x12!\n\x19proxy_dispatch_backoff_ms\x18\x0e \x01(\x03\x12\x0b\n\x03\x65nv\x18\x0f \x03(\t\x12!\n\x19\x66use_max_read_ahead_bytes\x18\x10 \x01(\x03\"\x94\x01\n\x0e\x46ileEntryBegin\x12\x0c\n\x04path\x18\x01 \x01(\t\x12 \n\x04type\x18\x02 \x01(\x0e\x32\x12.afsletpb.FileType\x12\x0c\n\x04mode\x18\x03 \x01(\r\x12\x0b\n\x03uid\x18\x04 \x01(\r\x12\x0b\n\x03gid\x18\x05 \x01(\r\x12\x12\n\nmtime_unix\x18\x06 \x01(\x03\x12\x16\n\x0esymlink_target\x18\x07 \x01(\t\"\x19\n\tFileChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x0e\n\x0c\x46ileEntryEnd\"\x81\x02\n\x0f\x45xecuteResponse\x12#\n\x03log\x18\x01 \x01(\x0b\x32\x14.afsletpb.LogMessageH\x00\x12\"\n\x06result\x18\x02 \x01(\x0b\x32\x10.afsletpb.ResultH\x00\x12)\n\ntar_header\x18\x03 \x01(\x0b\x32\x13.afsletpb.TarHeaderH\x00\x12\'\n\ttar_chunk\x18\x04 \x01(\x0b\x32\x12.afsletpb.TarChunkH\x00\x12\x1e\n\x04\x64one\x18\x05 \x01(\x0b\x32\x0e.afsletpb.DoneH\x00\x12&\n\x08\x61\x63\x63\x65pted\x18\x06 \x01(\x0b\x32\x12.afsletpb.AcceptedH\x00\x42\t\n\x07payload\"\x1c\n\x08\x41\x63\x63\x65pted\x12\x10\n\x08\x61\x63\x63\x65pted\x18\x01 \x01(\x08\"-\n\nLogMessage\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\";\n\x06Result\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x11\n\texit_code\x18\x02 \x01(\x05\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"\x19\n\tTarHeader\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x18\n\x08TarChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x06\n\x04\x44one\"\x19\n\x17GetRuntimeStatusRequest\"\xd2\x01\n\x18GetRuntimeStatusResponse\x12\x1a\n\x12running_containers\x18\x01 \x01(\x03\x12\x17\n\x0flimit_cpu_cores\x18\x02 \x01(\x03\x12\x17\n\x0flimit_memory_mb\x18\x03 \x01(\x03\x12\x16\n\x0eused_cpu_cores\x18\x04 \x01(\x03\x12\x16\n\x0eused_memory_mb\x18\x05 \x01(\x03\x12\x1b\n\x13\x61vailable_cpu_cores\x18\x06 \x01(\x03\x12\x1b\n\x13\x61vailable_memory_mb\x18\x07 \x01(\x03*H\n\x08\x46ileType\x12\x11\n\rFILE_TYPE_DIR\x10\x00\x12\x12\n\x0e\x46ILE_TYPE_FILE\x10\x01\x12\x15\n\x11\x46ILE_TYPE_SYMLINK\x10\x02\x32\xa7\x01\n\x06\x41\x66slet\x12\x42\n\x07\x45xecute\x12\x18.afsletpb.ExecuteRequest\x1a\x19.afsletpb.ExecuteResponse(\x01\x30\x01\x12Y\n\x10GetRuntimeStatus\x12!.afsletpb.GetRuntimeStatusRequest\x1a\".afsletpb.GetRuntimeStatusResponseB.Z,github.com/reyoung/afs/pkg/afsletpb;afsletpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'api.afslet.v1.afslet_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/reyoung/afs/pkg/afsletpb;afsletpb'
  _globals['_FILETYPE']._serialized_start=1516
  _globals['_FILETYPE']._serialized_end=1588
  _globals['_EXECUTEREQUEST']._serialized_start=41
  _globals['_EXECUTEREQUEST']._serialized_end=244
  _globals['_STARTREQUEST']._serialized_start=247
  _globals['_STARTREQUEST']._serialized_end=621
  _globals['_FILEENTRYBEGIN']._serialized_start=624
  _globals['_FILEENTRYBEGIN']._serialized_end=772
  _globals['_FILECHUNK']._serialized_start=774
  _globals['_FILECHUNK']._serialized_end=799
  _globals['_FILEENTRYEND']._serialized_start=801
  _globals['_FILEENTRYEND']._serialized_end=815
  _globals['_EXECUTERESPONSE']._serialized_start=818
  _globals['_EXECUTERESPONSE']._serialized_end=1075
  _globals['_ACCEPTED']._serialized_start=1077
  _globals['_ACCEPTED']._serialized_end=1105
  _globals['_LOGMESSAGE']._serialized_start=1107
  _globals['_LOGMESSAGE']._serialized_end=1152
  _globals['_RESULT']._serialized_start=1154
  _globals['_RESULT']._serialized_end=1213
  _globals['_TARHEADER']._serialized_start=1215
  _globals['_TARHEADER']._serialized_end=1240
  _globals['_TARCHUNK']._serialized_start=1242
  _globals['_TARCHUNK']._serialized_end=1266
  _globals['_DONE']._serialized_start=1268
  _globals['_DONE']._serialized_end=1274
  _globals['_GETRUNTIMESTATUSREQUEST']._serialized_start=1276
  _globals['_GETRUNTIMESTATUSREQUEST']._serialized_end=1301
  _globals['_GETRUNTIMESTATUSRESPONSE']._serialized_start=1304
  _globals['_GETRUNTIMESTATUSRESPONSE']._serialized_end=1514
  _globals['_AFSLET']._serialized_start=1591
  _globals['_AFSLET']._serialized_end=1758
# @@protoc_insertion_point(module_scope)
